"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("@typescript-eslint/types");
const util_1 = require("../util");
exports.default = (0, util_1.createRule)({
    defaultOptions: [{ allowConditionalRequire: false }],
    name: 'no-require',
    meta: {
        type: 'suggestion',
        docs: {
            description: (0, util_1.ruleMessageTemplate)({
                why: 'Prefer ESM import/export syntax over CommonJS module.exports and require().',
                linterMessage: 'ESM is the standard for JavaScript modules, and is also much more readable compared to require().',
            }),
            recommended: false,
        },
        messages: {
            importInsteadOfRequire: (0, util_1.ruleMessageTemplate)({
                why: 'TypeScript is recommending to use the ESM (ECMAScript Modules)',
                linterMessage: 'Expected "import" instead of "require()"',
            }),
        },
        schema: [
            {
                type: 'object',
                properties: {
                    // example: 'if (typeof window !== "undefined") require("x")'
                    allowConditionalRequire: { type: 'boolean' },
                },
                additionalProperties: false,
            },
        ],
    },
    create(context) {
        // eslint-disable-next-line @typescript-eslint/strict-boolean-expressions
        const { allowConditionalRequire = false } = context.options[0] || {};
        return {
            CallExpression: function (node) {
                if (!isRequire(node))
                    return;
                if (!hasOneArgument(node))
                    return;
                if (allowConditionalRequire && isConditional(node))
                    return;
                context.report({
                    node: node.callee,
                    messageId: 'importInsteadOfRequire',
                });
            },
        };
    },
});
/** Checks that the 'require()' call has only 1 argument (TS shows error when more than 1 argument is provided) */
function hasOneArgument(node) {
    return node.arguments.length === 1;
}
/** Checks if the callee (what is being called) is 'require()' */
function isRequire(node) {
    return (node.callee.type === types_1.AST_NODE_TYPES.Identifier &&
        node.callee.name === 'require');
}
/** Checks for a conditional statement.
 * example: 'if (true) require("x")'
 */
function isConditional(node) {
    if (node.type === types_1.AST_NODE_TYPES.IfStatement ||
        node.type === types_1.AST_NODE_TYPES.TryStatement ||
        node.type === types_1.AST_NODE_TYPES.LogicalExpression ||
        node.type === types_1.AST_NODE_TYPES.ConditionalExpression)
        return true;
    if (node.parent)
        return isConditional(node.parent);
    return false;
}
//# sourceMappingURL=no-require.js.map